@@ -4,7 +4,8 @@
 #define __GUILE_SDL_H
 
 /*
- * Copyright (C) 2003, 2004, 2005, 2007, 2008, 2009, 2011 Thien-Thi Nguyen
+ * Copyright (C) 2003, 2004, 2005, 2007, 2008,
+ *   2009, 2011, 2013 Thien-Thi Nguyen
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,6 +36,7 @@
    Let's see how long this insufferable anti-portability attitude
    stands in the Real World!  :-D  */
 #include <stdint.h>
+#include <stdbool.h>
 #include <SDL/SDL.h>
 
 
@@ -53,172 +55,11 @@
 /* Guile.  */
 
 #include <libguile.h>
+#include "snuggle/level.h"
+#include "snuggle/humdrum.h"
 
 #define PERMANENT  scm_permanent_object
 
-#if  defined (SCM_MAJOR_VERSION)                                \
-  && defined (SCM_MINOR_VERSION)                                \
-  && (2 == SCM_MAJOR_VERSION                                    \
-      || (1 == SCM_MAJOR_VERSION && 8 <= SCM_MINOR_VERSION))
-#define GI_LEVEL_NOT_YET_1_8  0
-#else
-#define GI_LEVEL_NOT_YET_1_8  1
-#endif
-
-#if GI_LEVEL_NOT_YET_1_8
-#include <guile/gh.h>
-#define NULLP(obj)        (gh_null_p (obj))
-#define PAIRP(obj)        (gh_pair_p (obj))
-#define SYMBOLP(obj)      (gh_symbol_p (obj))
-#define INTEGERP(obj)      NOT_FALSEP (scm_integer_p (obj))
-#define VECTORP            gh_vector_p
-#define STRINGP            gh_string_p
-#define BOOLEAN            gh_bool2scm
-#define NUM_INT            gh_int2scm
-#define NUM_LONG           gh_long2scm
-#define NUM_ULONG          gh_ulong2scm
-#define SYMBOL             gh_symbol2scm
-#define STRING             gh_str02scm
-#define BSTRING            gh_str2scm
-#define C_BOOL             gh_scm2bool
-#define C_CHAR             gh_scm2char
-#define C_INT              gh_scm2int
-#define C_LONG             gh_scm2long
-#define C_ULONG            gh_scm2ulong
-#define C_DOUBLE           gh_scm2double
-#define EQ                 gh_eq_p
-#define CONS               gh_cons
-#define CAR                gh_car
-#define CAAR               gh_caar
-#define CDR                gh_cdr
-#define SETCAR             gh_set_car_x
-#define VECLENGTH          gh_vector_length
-#define UVECLENGTH         gh_uniform_vector_length
-#define VECREF(vec,idx)    (SCM_VELTS (vec)[idx])
-#define CALL0              gh_call0
-#define CALL1              gh_call1
-#define CALL2              gh_call2
-#define CALL3              gh_call3
-#define GC_PROTECT         scm_protect_object
-#define GC_UNPROTECT       scm_unprotect_object
-#define LOOKUP             gh_lookup
-#define DEFINE_PUBLIC      gh_define
-#define MODULE_LOOKUP      gh_module_lookup
-#else
-#define NULLP(obj)        (scm_is_null (obj))
-#define PAIRP(obj)        (scm_is_true (scm_pair_p (obj)))
-#define SYMBOLP(obj)      (scm_is_symbol (obj))
-#define INTEGERP           scm_is_integer
-#define VECTORP            scm_is_vector
-#define STRINGP            scm_is_string
-#define BOOLEAN            scm_from_bool
-#define NUM_INT            scm_from_int
-#define NUM_LONG           scm_from_long
-#define NUM_ULONG          scm_from_ulong
-#define SYMBOL             scm_from_locale_symbol
-#define STRING             scm_from_locale_string
-#define BSTRING            scm_from_locale_stringn
-#define C_BOOL             scm_to_bool
-#define C_CHAR(c)          C_INT (scm_char_to_integer (c))
-#define C_INT              scm_to_int
-#define C_LONG             scm_to_long
-#define C_ULONG            scm_to_ulong
-#define C_DOUBLE           scm_to_double
-#define EQ                 scm_is_eq
-#define CONS               scm_cons
-#define CAR                scm_car
-#define CAAR               scm_caar
-#define CDR                scm_cdr
-#define SETCAR             scm_set_car_x
-#define VECLENGTH          scm_c_vector_length
-#define UVECLENGTH         scm_c_uniform_vector_length
-#define VECREF             scm_c_vector_ref
-#define CALL0              scm_call_0
-#define CALL1              scm_call_1
-#define CALL2              scm_call_2
-#define CALL3              scm_call_3
-#define GC_PROTECT         scm_gc_protect_object
-#define GC_UNPROTECT       scm_gc_unprotect_object
-#define LOOKUP(name)      (scm_variable_ref (scm_c_lookup (name)))
-#define DEFINE_PUBLIC(name,value)  do           \
-    {                                           \
-      scm_c_define (name, value);               \
-      scm_c_export (name, NULL);                \
-    }                                           \
-  while (0)
-#define MODULE_LOOKUP(module, name)                     \
-  scm_variable_ref (scm_c_module_lookup (module, name))
-#endif
-
-
-/* Abstractions for Scheme objects to C string conversion.  */
-
-typedef struct {
-  char *s;
-  size_t len;
-} range_t;
-
-#define RS(svar)    c ## svar .s
-#define RLEN(svar)  c ## svar .len
-
-#if GI_LEVEL_NOT_YET_1_8
-
-#define FINANGLABLE_SCHEME_STRING_FROM_SYMBOL(sym)      \
-  scm_string_copy (scm_symbol_to_string (sym))
-
-/* Coerce a Scheme (sub)string that is to be used in contexts where the
-   extracted C string is expected to be read-only and zero-terminated.  We
-   check this condition precisely instead of simply coercing all (sub)strings,
-   to avoid waste for those substrings that may in fact (by lucky accident)
-   already satisfy the condition.  */
-#define ROZT_X(x)                                       \
-  if (SCM_ROCHARS (x) [SCM_ROLENGTH (x)])               \
-    x = BSTRING (SCM_ROCHARS (x), SCM_ROLENGTH (x))
-
-#define _FINANGLE(svar,p1)  do                  \
-    {                                           \
-      if (p1)                                   \
-        ROZT_X (svar);                          \
-      RS (svar) = SCM_ROCHARS (svar);           \
-      RLEN (svar) = SCM_ROLENGTH (svar);        \
-    }                                           \
-  while (0)
-
-#define UNFINANGLE(svar)
-
-#else  /* !GI_LEVEL_NOT_YET_1_8 */
-
-#define FINANGLABLE_SCHEME_STRING_FROM_SYMBOL  scm_symbol_to_string
-
-#define REND(svar)          RS (svar) [RLEN (svar)]
-#define NUL_AT_END_X(svar)  REND (svar) = '\0'
-
-#define _FINANGLE(svar,p1)  do                                  \
-    {                                                           \
-      RS (svar) = scm_to_locale_stringn (svar, &RLEN (svar));   \
-      if (RS (svar))                                            \
-        {                                                       \
-          if (p1 && REND (svar))                                \
-            {                                                   \
-              RS (svar) = realloc (RS (svar), 1 + RLEN (svar)); \
-              NUL_AT_END_X (svar);                              \
-            }                                                   \
-        }                                                       \
-      else                                                      \
-        RS (svar) = strdup ("");                                \
-    }                                                           \
-  while (0)
-
-#define UNFINANGLE(svar)  free (RS (svar))
-
-#endif  /* !GI_LEVEL_NOT_YET_1_8 */
-
-/* Use ‘FINANGLE_RAW’ when the consumer of the C string takes full range
-   (start address plus length) info.  Otherwise, ‘FINANGLE’.  */
-
-#define FINANGLE_RAW(svar)  _FINANGLE (svar, 0)
-#define FINANGLE(svar)      _FINANGLE (svar, 1)
-
 
 #ifdef HAVE_GUILE_MODSUP_H
 #include <guile/modsup.h>
@@ -284,18 +125,6 @@ void scm_init_ ## frag ## _module (void) { func (); }
 #define EXACTLY_FALSEP(x)  (SCM_FALSEP (x))
 #define EXACTLY_TRUEP(x)   (EQ ((x), BOOL_TRUE))
 
-#define SET_FALSE(cvar) \
-  cvar = BOOL_FALSE
-
-#define SET_TRUE(cvar) \
-  cvar = BOOL_TRUE
-
-#define RETURN_FALSE \
-  return BOOL_FALSE
-
-#define RETURN_TRUE \
-  return BOOL_TRUE
-
 /* Argument validation.  */
 
 #define ASSERT_TYPE(condition,object,position,expected) \
@@ -341,7 +170,7 @@ void scm_init_ ## frag ## _module (void) { func (); }
 #define BOUNDP(x)    (! SCM_EQ_P (x, SCM_UNDEFINED))
 #define UNBOUNDP(x)    (SCM_EQ_P (x, SCM_UNDEFINED))
 
-#define UNBOUND_MEANS_FALSE(x)  if (UNBOUNDP (x)) SET_FALSE (x)
+#define UNBOUND_MEANS_FALSE(x)  if (UNBOUNDP (x)) x = BOOL_FALSE
 
 #define IF_BOUND_ASSERT_INTEGER_COPY(name,pos,change)   \
   if (BOUNDP (name) && NOT_FALSEP (name))               \
@@ -355,58 +184,81 @@ void scm_init_ ## frag ## _module (void) { func (); }
 
 /* Enums and flagstashes.  */
 
-typedef union {
-  const char const *rozt;
-  SCM symbol;
-} aka_t;
+struct symset {
+  const size_t      count;
+  const char const *name;
+  const uint8_t    *pool;
+};
 
 typedef struct {
-  const long int value;
-  aka_t aka;
-} valaka_t;
+  const long const *val;
+  SCM *linear;
+  SCM table;
+  int classic:1;                        /* boolean */
+  int offset:1;                         /* boolean */
+  struct symset ss;
+} kp_t;
 
-#define VALAKA(x)  { x, { #x } }
+typedef struct flagstash {
+  const unsigned long const *val;
+  SCM *linear;
+  SCM ht;
+  unsigned long full;
+  struct symset ss;
+} flagstash_t;
 
-#define enum_nick "SDL-enum"
+/* We use "caller constants" to reduce data motion.
+   Note also the ‘const’ in ‘DECLINIT_SYM2NUM_CC’.  */
 
-typedef SCM (define_enum_t) (const char *name, size_t count,
-                             valaka_t *backing);
+typedef struct sym2num_cc {
+  const SCM   stash;
+  const char *who;
+  const int   pos;
+} sym2num_cc_t;
 
-#define DEFINE_ENUM(name,backing)               \
-  btw->define_enum                              \
-  (name,                                        \
-   sizeof (backing) / sizeof (valaka_t),        \
-   backing)
+#define DECLINIT_SYM2NUM_CC(POS,STASH)          \
+  const sym2num_cc_t _CC_ ## POS =              \
+    {                                           \
+      .stash = STASH,                           \
+      .who = FUNC_NAME,                         \
+      .pos = POS                                \
+    }
 
-typedef long (enum2long_t) (SCM s_enum, SCM enum_type,
-                            int pos, const char *func);
+typedef struct {
+  SCM  *smob;
+  kp_t *kp;
+} kp_init_t;
+
+typedef void (register_kp_v_t) (size_t count, kp_init_t v[count]);
+
+#define REGISTER_KP_V(var) \
+  btw->register_kp_v (sizeof (var) / sizeof (kp_init_t), var)
+
+typedef long (enum2long_t) (const sym2num_cc_t *cc, SCM obj);
 
 typedef SCM (long2enum_t) (long value, SCM enum_type);
 
-#define GSDL_ENUM2LONG(enums,table,pos) \
-  btw->enum2long ((enums), (table), (pos), FUNC_NAME)
+#define ENUM2LONG(POS,OBJECT) \
+  btw->enum2long (& _CC_ ## POS, OBJECT)
 
 /* flags (constants typically used as a logical or'ed group) */
 
-#define flagstash_nick "SDL-flagstash"
-
-typedef unsigned long (flags2ulong_t) (SCM flags, SCM table,
-                                       int pos, const char *func);
+typedef unsigned long (flags2ulong_t) (const sym2num_cc_t *cc, SCM obj);
 
 typedef SCM (ulong2flags_t) (unsigned long value, SCM stash);
 
-#define GSDL_FLAGS2ULONG(flags,table,pos) \
-  btw->flags2ulong ((flags), (table), (pos), FUNC_NAME)
+#define FLAGS2ULONG(POS,OBJECT) \
+  btw->flags2ulong (& _CC_ ## POS, OBJECT)
 
-typedef struct flagstash {
-  const char const *name;
-  const size_t total;
-  const unsigned long const *val;
-  aka_t *aka;
-  SCM ht;
-} flagstash_t;
+typedef struct {
+  SCM         *smob;
+  flagstash_t *stash;
+} kf_init_t;
 
-typedef SCM (make_flagstash_t) (flagstash_t *stash);
+typedef void (register_kf_v_t) (size_t count, kf_init_t v[count]);
+
+#define REGISTER_KF_V(var) \
+  btw->register_kf_v (sizeof (var) / sizeof (kf_init_t), var)
 
 /* Symbols.  */
 
@@ -420,34 +272,7 @@ typedef SCM (make_flagstash_t) (flagstash_t *stash);
 
 /* Smobs.  */
 
-#if GI_LEVEL_NOT_YET_1_8
-
-#define DEFSMOB(tagvar,name,m,f,p)                              \
-  tagvar = scm_make_smob_type_mfpe (name, 0, m, f, p, NULL)
-
-#define GCMALLOC(sz,what)    scm_must_malloc (sz, what)
-#define GCFREE(ptr,what)     scm_must_free (ptr)
-#define GCRV(ptr)            sizeof (*ptr)
-
-#else  /* !GI_LEVEL_NOT_YET_1_8 */
-
-#define DEFSMOB(tagvar,name,m,f,p)  do          \
-    {                                           \
-      tagvar = scm_make_smob_type (name, 0);    \
-      if (NULL != m)                            \
-        scm_set_smob_mark (tagvar, m);          \
-      if (NULL != f)                            \
-        scm_set_smob_free (tagvar, f);          \
-      if (NULL != p)                            \
-        scm_set_smob_print (tagvar, p);         \
-    }                                           \
-  while (0)
-
-#define GCMALLOC(sz,what)    scm_gc_malloc (sz, what)
-#define GCFREE(ptr,what)     scm_gc_free (ptr, sizeof (*ptr), what)
-#define GCRV(ptr)            0
-
-#endif  /* !GI_LEVEL_NOT_YET_1_8 */
+#include "snuggle/defsmob.h"
 
 /* useful type-checking for smobs */
 #define ASSERT_SMOB(smob, lpre, which)                          \
@@ -474,7 +299,7 @@ typedef SCM (make_flagstash_t) (flagstash_t *stash);
 #define DECLARE_PF(ACTUAL)                      \
   typedef struct {                              \
     SDL_ ## ACTUAL *object;                     \
-    int internalp;                              \
+    bool internalp;                             \
   } PF_ ## ACTUAL
 
 struct obtw
@@ -485,12 +310,17 @@ struct obtw
   SCM video_flags;
   /* Flags used in various sdl{surface,video}.c funcs.  */
 
-  make_flagstash_t *make_flagstash;
+  SCM event_state_enum;
+  SCM updn_enum;
+  /* Used in various sdl{event,joystick}.c funcs.  */
+
   flags2ulong_t *flags2ulong;
   ulong2flags_t *ulong2flags;
-  define_enum_t *define_enum;
   enum2long_t *enum2long;
   long2enum_t *long2enum;
+
+  register_kp_v_t *register_kp_v;
+  register_kf_v_t *register_kf_v;
 };
 
 #if 1 == GUILE_SDL_OPTIONAL_MODULE
@@ -537,7 +367,7 @@ DECLARE_PF (Surface);
     if ((__p = (x)))                            \
       SCM_RETURN_NEWSMOB (tag, __p);            \
     else                                        \
-      RETURN_FALSE;                             \
+      return BOOL_FALSE;                        \
   } while (0)
 
 #define RETURN_NEW_PF_OR_FALSE(ACTUAL,LOCALPREFIX,INTERNALP,X)  do      \
@@ -553,7 +383,7 @@ DECLARE_PF (Surface);
           SCM_RETURN_NEWSMOB (LOCALPREFIX ## _tag, pf);                 \
         }                                                               \
       else                                                              \
-        RETURN_FALSE;                                                   \
+        return BOOL_FALSE;                                              \
     }                                                                   \
   while (0)
 
@@ -562,8 +392,8 @@ DECLARE_PF (Surface);
 
 #define RETURN_NEW_COLOR(x)         NEWSMOB_OR_FALSE (color_tag, x)
 #define RETURN_NEW_RECT(x)          NEWSMOB_OR_FALSE (rect_tag, x)
-#define RETURN_NEW_SURFACE(x)       RETURN_PF_SURFACE (x, 0)
-#define RETURN_INT_SURFACE(x)       RETURN_PF_SURFACE (x, 1)
+#define RETURN_NEW_SURFACE(x)       RETURN_PF_SURFACE (x, false)
+#define RETURN_INT_SURFACE(x)       RETURN_PF_SURFACE (x, true)
 #define RETURN_NEW_PIXEL_FORMAT(x)  NEWSMOB_OR_FALSE (pixel_format_tag, x)
 
 /* Lots of SDL functions return 0 for true, -1 otherwise.  */
@@ -590,73 +420,17 @@ DECLARE_PF (Surface);
 #define RETURN_0STR(exp) \
   return STRING (exp)
 
-/* Return various lists of various length.  */
-
-#define RETURN_LIST0 \
-  return SCM_EOL
-
-#define _rv_PUSH(x) \
-  _rv = CONS ((x), _rv)
-
-#define RETURN_LIST2(a,b)                       \
-  do {                                          \
-    SCM _rv = SCM_EOL;                          \
-    _rv_PUSH (b);                               \
-    _rv_PUSH (a);                               \
-    return _rv;                                 \
-  } while (0)
-
-#define RETURN_LIST3(a,b,c)                     \
-  do {                                          \
-    SCM _rv = SCM_EOL;                          \
-    _rv_PUSH (c);                               \
-    _rv_PUSH (b);                               \
-    _rv_PUSH (a);                               \
-    return _rv;                                 \
-  } while (0)
-
-#define RETURN_LIST4(a,b,c,d)                   \
-  do {                                          \
-    SCM _rv = SCM_EOL;                          \
-    _rv_PUSH (d);                               \
-    _rv_PUSH (c);                               \
-    _rv_PUSH (b);                               \
-    _rv_PUSH (a);                               \
-    return _rv;                                 \
-  } while (0)
-
-#define RETURN_LIST5(a,b,c,d,e)                 \
-  do {                                          \
-    SCM _rv = SCM_EOL;                          \
-    _rv_PUSH (e);                               \
-    _rv_PUSH (d);                               \
-    _rv_PUSH (c);                               \
-    _rv_PUSH (b);                               \
-    _rv_PUSH (a);                               \
-    return _rv;                                 \
-  } while (0)
-
-#define RETURN_LIST11(a,b,c,d,e,f,g,h,i,j,k)    \
-  do {                                          \
-    SCM _rv = SCM_EOL;                          \
-    _rv_PUSH (k);                               \
-    _rv_PUSH (j);                               \
-    _rv_PUSH (i);                               \
-    _rv_PUSH (h);                               \
-    _rv_PUSH (g);                               \
-    _rv_PUSH (f);                               \
-    _rv_PUSH (e);                               \
-    _rv_PUSH (d);                               \
-    _rv_PUSH (c);                               \
-    _rv_PUSH (b);                               \
-    _rv_PUSH (a);                               \
-    return _rv;                                 \
-  } while (0)
-
-/* Throws.  */
-
-#define THROW_NOT_YET_IMPLEMENTED \
-  SCM_MISC_ERROR ("not yet implemented (sorry)", SCM_EOL)
+/* Like ‘RETURN_0STR’, but if ‘exp’ evaluates to ‘NULL’,
+   return ‘#f’ instead.  */
+#define RETURN_0STR_OR_FALSE(exp)  do           \
+      {                                         \
+        const char *__p = (exp);                \
+                                                \
+        return __p                              \
+          ? STRING (__p)                        \
+          : BOOL_FALSE;                         \
+      }                                         \
+    while (0)
 
 /* Whole functions.  */
 
@@ -710,8 +484,9 @@ PRIMPROC (cname, sname, 2, 0, 0, (SCM lpre, SCM value),                 \
           " to @var{value}, a symbol or integer.")                      \
 {                                                                       \
   const char *FUNC_NAME = s_ ## cname;                                  \
+  DECLINIT_SYM2NUM_CC (1, etype);                                       \
   ASSERT_SMOB (lpre, lpre, 1);                                          \
-  SMOBF (lpre, ctype, field) = GSDL_ENUM2LONG (value, etype, 1);        \
+  SMOBF (lpre, ctype, field) = ENUM2LONG (1, value);                    \
   RETURN_UNSPECIFIED;                                                   \
 }
 
@@ -733,11 +508,22 @@ PRIMPROC (cname, sname, 2, 0, 0, (SCM lpre, SCM value),                 \
           " to @var{value}, a (possibly empty) list of symbols.")       \
 {                                                                       \
   const char *FUNC_NAME = s_ ## cname;                                  \
+  DECLINIT_SYM2NUM_CC (2, stash);                                       \
   ASSERT_SMOB (lpre, lpre, 1);                                          \
-  SMOBF (lpre, ctype, field) = GSDL_FLAGS2ULONG (value, stash, 2);      \
+  SMOBF (lpre, ctype, field) = FLAGS2ULONG (2, value);                  \
   RETURN_UNSPECIFIED;                                                   \
 }
 
 #endif /* !defined (__GUILE_SDL_H) */
 
+
+/* Miscellaneous idioms.  */
+
+#define CSTATE_FROM_SETTING(setting)            \
+  (UNBOUNDP (setting)                           \
+   ? SDL_QUERY                                  \
+   : (EXACTLY_FALSEP (setting)                  \
+      ? SDL_IGNORE                              \
+      : SDL_ENABLE))
+
 /* guile-sdl.h ends here */
