@@ -1,6 +1,7 @@
 ;;; misc-utils.scm --- grab bag of utility procs
 
-;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2011 Thien-Thi Nguyen
+;; Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009,
+;;               2011, 2013 Thien-Thi Nguyen
 ;;
 ;; This is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -275,19 +276,18 @@
                 s)))
         (else (conv surface))))
 
-;; Arrange to ignore all event types except those in @var{ls} (zero or
-;; more symbols from @code{event-types}).  As a special case, if @var{ls}
+;; Arrange to ignore all event types except those in @var{types}
+;; (@pxref{event-type enums}).  As a special case, if @var{types}
 ;; is @code{#f}, arrange to not ignore any event types (all are enabled).
-;; @xref{Enums and Constants}.
 ;;
-(define (ignore-all-event-types-except . ls)
-  (let ((proc (if (and (not (null? ls)) (not (car ls)))
+(define (ignore-all-event-types-except . types)
+  (let ((proc (if (and (not (null? types)) (not (car types)))
                   (lambda (type)
-                    (SDL:event-state type 'SDL_ENABLE))
+                    (SDL:event-type-handling type #t))
                   (lambda (type)
-                    (or (memq type ls)
-                        (SDL:event-state type 'SDL_IGNORE))))))
-    (for-each proc (SDL:enumstash-enums SDL:event-types))))
+                    (or (memq type types)
+                        (SDL:event-type-handling type #f))))))
+    (for-each proc (map car (SDL:kotk 'event-type)))))
 
 ;; Return three values, each a thunk, that can be used to loop for
 ;; @var{sec} seconds, blitting onto @var{realized} at @var{location} (a
@@ -347,7 +347,7 @@
                        (= alpha new-a))
                 (begin (set! alpha new-a)
                        (SDL:blit-surface one #f base loc)
-                       (SDL:set-alpha! two 'SDL_SRCALPHA alpha)
+                       (SDL:surface-alpha! two alpha)
                        (SDL:blit-surface two #f base loc)
                        (show)
                        (or (< alpha 255)
