@@ -1,6 +1,7 @@
 /* sdlmixer.c --- SDL_mixer for Guile
  *
- * Copyright (C) 2003, 2004, 2005, 2007, 2008, 2009, 2011 Thien-Thi Nguyen
+ * Copyright (C) 2003, 2004, 2005, 2007, 2008,
+ *   2009, 2011, 2013 Thien-Thi Nguyen
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,17 +22,15 @@
 #define GUILE_SDL_OPTIONAL_MODULE  1
 #include "guile-sdl.h"
 #include <SDL/SDL_mixer.h>
+#include "snuggle/finangle.h"
+#include "b-values.h"
 
 IMPORT_MODULE (sdlsup, "(sdl sdl)");
 SELECT_MODULE_VAR (obtw, sdlsup, "%%Guile-SDL-obtw");
 
 
 static SCM fading_status_enum;
-static valaka_t fading_status_eback[] = {
-  VALAKA (MIX_NO_FADING),
-  VALAKA (MIX_FADING_OUT),
-  VALAKA (MIX_FADING_IN)
-};
+#include "k/fading.c"
 
 #define RETURN_FADINGSTATUS(x)  return btw->long2enum ((x), fading_status_enum)
 
@@ -113,6 +112,30 @@ new number of allocated channels.  */)
 #undef FUNC_NAME
 }
 
+
+PRIMPROC
+(device_ffc, "device-ffc", 0, 0, 0,
+ (void),
+ doc: /***********
+Return audio device parameters as three values: @code{frequency} (Hz),
+@code{format} (number of bits) and @code{channels} (number of
+allocated channels).  */)
+{
+#define FUNC_NAME s_device_ffc
+  int freq, channels;
+  Uint16 format;
+
+  if (! Mix_QuerySpec (&freq, &format, &channels))
+    SCM_MISC_ERROR ("audio not open", SCM_EOL);
+
+  RETURN_VALUES3
+    (NUM_LONG (freq),
+     NUM_LONG (format),
+     NUM_LONG (channels));
+#undef FUNC_NAME
+}
+
+
 DECLARE_SIMPLE_SYM (freq);
 DECLARE_SIMPLE_SYM (format);
 DECLARE_SIMPLE_SYM (channels);
@@ -121,6 +144,9 @@ PRIMPROC
 (mix_query_spec, "query-spec", 0, 0, 0,
  (void),
  doc: /***********
+NB: This procedure is obsoleted by @code{device-ffc}
+and @strong{will be removed} after 2013-12-31.
+
 Return audio device parameters as an alist, or @code{#f}
 if the audio has not yet been opened.
 Keys are @code{freq} (frequency), @code{format},
@@ -131,9 +157,9 @@ and @code{channels} (the number of allocated channels).  */)
   Uint16 format;
 
   if (! Mix_QuerySpec (&freq, &format, &channels))
-    RETURN_FALSE;
+    return BOOL_FALSE;
 
-  RETURN_LIST3 (CONS (SYM (freq), NUM_LONG (freq)),
+  return LIST3 (CONS (SYM (freq), NUM_LONG (freq)),
                 CONS (SYM (format), NUM_LONG (format)),
                 CONS (SYM (channels), NUM_LONG (channels)));
 #undef FUNC_NAME
@@ -142,19 +168,19 @@ and @code{channels} (the number of allocated channels).  */)
 
 PRIMPROC
 (mix_load_music, "load-music", 1, 0, 0,
- (SCM file),
+ (SCM filename),
  doc: /***********
-Load a wave or a music (.mod .s3m .it .xm) @var{file}.
-Return a handle to it.  */)
+Load music data (.mod .s3m .it .xm) from @var{filename}.
+Return a new music object if successful, otherwise @code{#f}.  */)
 {
 #define FUNC_NAME s_mix_load_music
-  range_t cfile;
+  range_t cfilename;
   Mix_Music *rv;
 
-  ASSERT_STRING (file, 1);
-  FINANGLE (file);
-  rv = Mix_LoadMUS (RS (file));
-  UNFINANGLE (file);
+  ASSERT_STRING (filename, 1);
+  FINANGLE (filename);
+  rv = Mix_LoadMUS (RS (filename));
+  UNFINANGLE (filename);
 
   RETURN_NEW_MUSIC (rv);
 #undef FUNC_NAME
@@ -163,18 +189,19 @@ Return a handle to it.  */)
 
 PRIMPROC
 (mix_load_wave, "load-wave", 1, 0, 0,
- (SCM file),
+ (SCM filename),
  doc: /***********
-Load a wave @var{file}. Return a handle to it.  */)
+Load wave data from @var{filename}.
+Return a new audio object if succesful, otherwise @code{#f}.  */)
 {
 #define FUNC_NAME s_mix_load_wave
-  range_t cfile;
+  range_t cfilename;
   Mix_Chunk *rv;
 
-  ASSERT_STRING (file, 1);
-  FINANGLE (file);
-  rv = Mix_LoadWAV (RS (file));
-  UNFINANGLE (file);
+  ASSERT_STRING (filename, 1);
+  FINANGLE (filename);
+  rv = Mix_LoadWAV (RS (filename));
+  UNFINANGLE (filename);
 
   RETURN_NEW_AUDIO (rv);
 #undef FUNC_NAME
@@ -592,8 +619,8 @@ PRIMPROC
 (mix_fading_music, "fading-music", 0, 0, 0,
  (void),
  doc: /***********
-Return the fading status of the music.
-@xref{Enums and Constants}.  */)
+Return the fading status of the music, one of the symbols:
+@code{no}, @code{out}, @code{in}.  */)
 {
 #define FUNC_NAME s_mix_fading_music
   RETURN_FADINGSTATUS (Mix_FadingMusic ());
@@ -605,9 +632,9 @@ PRIMPROC
 (mix_fading_channel, "fading-channel", 0, 1, 0,
  (SCM which),
  doc: /***********
-Return the fading status of the default channel.
-Optional arg @var{which} selects which channel to check.
-@xref{Enums and Constants}.  */)
+Return the fading status (a symbol, see @code{fading-music})
+of the default channel.
+Optional arg @var{which} selects which channel to check.  */)
 {
 #define FUNC_NAME s_mix_fading_channel
   int cwhich = -1;
@@ -766,16 +793,22 @@ PRIMPROC
  (SCM command),
  doc: /***********
 Stop music and set external music playback command
-to @var{command}, a string.  */)
+to @var{command}, a string.  As a special case, if @var{command}
+is @code{#f}, arrange to use internal playback, instead.  */)
 {
 #define FUNC_NAME s_mix_set_music_cmd
   range_t ccommand;
   int rv;
 
-  ASSERT_STRING (command, 1);
-  FINANGLE (command);
-  rv = Mix_SetMusicCMD (RS (command));
-  UNFINANGLE (command);
+  if (EXACTLY_FALSEP (command))
+    rv = Mix_SetMusicCMD (NULL);
+  else
+    {
+      ASSERT_STRING (command, 1);
+      FINANGLE (command);
+      rv = Mix_SetMusicCMD (RS (command));
+      UNFINANGLE (command);
+    }
   RETURN_INT (rv);
 #undef FUNC_NAME
 }
@@ -900,8 +933,13 @@ init_module (void)
 
   btw = UNPACK_POINTER (CALL0 (obtw));
 
-  /* enums */
-  fading_status_enum = DEFINE_ENUM ("fading-status", fading_status_eback);
+  {
+    kp_init_t allp[] = {
+      { &fading_status_enum, &fading_kp }
+    };
+
+    REGISTER_KP_V (allp);
+  }
 }
 
 MOD_INIT_LINK_THUNK ("sdl mixer", sdl_mixer, init_module)
