@@ -1,6 +1,7 @@
 /* sdlvideo.c --- SDL Video functions for Guile
  *
- * Copyright (C) 2003, 2004, 2005, 2007, 2009, 2011, 2012 Thien-Thi Nguyen
+ * Copyright (C) 2003, 2004, 2005, 2007, 2009,
+ *   2011, 2012, 2013 Thien-Thi Nguyen
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,9 +22,11 @@
 #include "guile-sdl.h"
 #include <stdio.h>
 #include <SDL/SDL_image.h>
+#include "snuggle/finangle.h"
 #include "b-uv.h"
+#include "b-values.h"
 
-#if GI_LEVEL_NOT_YET_1_8
+#if ! GI_LEVEL_1_8
 IMPORT_SRFI4 ();
 SELECT_MODULE_VAR (mk_u16v, srfi4, "make-u16vector");
 SELECT_MODULE_VAR (u16v_x, srfi4, "u16vector-set!");
@@ -32,10 +35,6 @@ SELECT_UVEC_PREDICATE (u16);
 #define scm_u8vector_p(obj)           CALL1 (u8v_p, obj)
 #define scm_u16vector_p(obj)          CALL1 (u16v_p, obj)
 #define scm_make_u16vector(len,fill)  CALL2 (mk_u16v, len, fill)
-
-/* For backward compatability (EVENTUALLY ZONKABLE).  */
-DEFINE_ANY_TO (u8)
-DEFINE_ANY_TO (u16)
 #endif
 
 DEFINE_STRUCT_AND_COPY_FUNC (u8, Uint8)
@@ -61,53 +60,9 @@ DEFINE_STRUCT_AND_COPY_FUNC (u16, Uint16)
   (SCM_SMOB_PREDICATE (rect_tag, x))
 
 
-static SCM gl_enums;
-static valaka_t gl_eback[] = {
-  VALAKA (SDL_GL_RED_SIZE),
-  VALAKA (SDL_GL_GREEN_SIZE),
-  VALAKA (SDL_GL_BLUE_SIZE),
-  VALAKA (SDL_GL_ALPHA_SIZE),
-  VALAKA (SDL_GL_BUFFER_SIZE),
-  VALAKA (SDL_GL_DOUBLEBUFFER),
-  VALAKA (SDL_GL_DEPTH_SIZE),
-  VALAKA (SDL_GL_STENCIL_SIZE),
-  VALAKA (SDL_GL_ACCUM_RED_SIZE),
-  VALAKA (SDL_GL_ACCUM_GREEN_SIZE),
-  VALAKA (SDL_GL_ACCUM_BLUE_SIZE),
-  VALAKA (SDL_GL_ACCUM_ALPHA_SIZE)
-};
-
 static SCM palette_flags;
 static SCM gsdl_overlay_formats;
-
-PRIMPROC
-(get_video_flags, "flagstash:video", 0, 0, 0, (),
- doc: /***********
-Return the flagstash object for video flags.
-@xref{Enums and Constants}.  */)
-{
-  return btw->video_flags;
-}
-
-PRIMPROC
-(get_palette_flags, "flagstash:palette", 0, 0, 0, (),
- doc: /***********
-Return the flagstash object for palette flags.
-@xref{Enums and Constants}.  */)
-{
-  return palette_flags;
-}
-
-PRIMPROC
-(get_overlay_formats, "flagstash:overlay", 0, 0, 0, (),
- doc: /***********
-Return the flagstash object for overlay flags.
-(Actually, these are "common overlay formats", not flags.)
-@xref{Enums and Constants}.  */)
-{
-  return gsdl_overlay_formats;
-}
-
+static SCM grab_modes;
 
 
 /* extended SDL_* structures */
@@ -132,8 +87,8 @@ static long overlay_tag;
 #define RETURN_PF_CURSOR(x,REFP)                        \
   RETURN_NEW_PF_OR_FALSE (Cursor, cursor, REFP, x)
 
-#define RETURN_NEW_CURSOR(x)    RETURN_PF_CURSOR (x, 0)
-#define RETURN_INT_CURSOR(x)    RETURN_PF_CURSOR (x, 1)
+#define RETURN_NEW_CURSOR(x)    RETURN_PF_CURSOR (x, false)
+#define RETURN_INT_CURSOR(x)    RETURN_PF_CURSOR (x, true)
 #define RETURN_NEW_OVERLAY(x)   NEWSMOB_OR_FALSE (overlay_tag, x)
 
 /* smob functions */
@@ -236,6 +191,7 @@ Optional arg @var{display} specifies a surface to use
 instead of creating a new one.  */)
 {
 #define FUNC_NAME s_create_yuv_overlay
+  DECLINIT_SYM2NUM_CC (3, gsdl_overlay_formats);
   Uint32 cformat;
   SDL_Surface *cdisplay;
 
@@ -243,7 +199,7 @@ instead of creating a new one.  */)
   ASSERT_INTEGER (height, 2);
 
   if (SYMBOLP (format))
-    cformat = GSDL_FLAGS2ULONG (format, gsdl_overlay_formats, 3);
+    cformat = FLAGS2ULONG (3, format);
   else
     ASSERT_ULONG_COPY (format, 3);
 
@@ -283,6 +239,52 @@ DECLARE_SYM (blit_sw,      "blit-sw");
 DECLARE_SYM (blit_sw_CC,   "blit-sw-CC");
 DECLARE_SYM (blit_sw_A,    "blit-sw-A");
 DECLARE_SYM (blit_fill,    "blit-fill");
+
+PRIMPROC
+(video_cmf, "video-cmf", 0, 0, 0,
+ (void),
+ doc: /***********
+Return information about the video hardware as three values:
+@code{capabilities} (list of symbols), @code{memory} (integer),
+and @code{format} (pixel format object).  The @code{capabilities} are:
+
+@example
+hw-available
+wm-available
+blit-hw   blit-hw-CC   blit-hw-A
+blit-sw   blit-sw-CC   blit-sw-A
+blit-fill
+@end example  */)
+{
+#define FUNC_NAME s_video_cmf
+  SCM cap = SCM_EOL;
+  const SDL_VideoInfo *info = SDL_GetVideoInfo ();
+  SCM format;
+
+  SCM_NEWSMOB (format, pixel_format_tag, info->vfmt);
+
+#define TRY(bit)  if (info->bit) cap = CONS (SYM (bit), cap)
+
+  TRY (blit_fill);
+  TRY (blit_sw_A);
+  TRY (blit_sw_CC);
+  TRY (blit_sw);
+  TRY (blit_hw_A);
+  TRY (blit_hw_CC);
+  TRY (blit_hw);
+  TRY (wm_available);
+  TRY (hw_available);
+
+#undef TRY
+
+  RETURN_VALUES3
+    (cap,
+     NUM_ULONG (info->video_mem),
+     format);
+#undef FUNC_NAME
+}
+
+
 DECLARE_SYM (video_mem,    "video-mem");
 DECLARE_SIMPLE_SYM (vfmt);
 
@@ -290,6 +292,9 @@ PRIMPROC
 (get_video_info, "get-video-info", 0, 0, 0,
  (void),
  doc: /***********
+NB: This procedure is obsoleted by @code{video-cmf}
+and @strong{will be removed} after 2013-12-31.
+
 Return information about the video hardware as an alist.
 Keys are: @code{hw-available}, @code{wm-available},
 @code{bit-hw}, @code{blit-hw-CC}, @code{blit-hw-A},
@@ -299,21 +304,27 @@ Keys are: @code{hw-available}, @code{wm-available},
 #define FUNC_NAME s_get_video_info
   const SDL_VideoInfo *info = SDL_GetVideoInfo ();
   SCM format;
+  SCM rv = SCM_EOL;
 
   SCM_NEWSMOB (format, pixel_format_tag, info->vfmt);
 
-  RETURN_LIST11
-    (CONS (SYM (hw_available), BOOLEAN (info->hw_available)),
-     CONS (SYM (wm_available), BOOLEAN (info->wm_available)),
-     CONS (SYM (blit_hw),      BOOLEAN (info->blit_hw)),
-     CONS (SYM (blit_hw_CC),   BOOLEAN (info->blit_hw_CC)),
-     CONS (SYM (blit_hw_A),    BOOLEAN (info->blit_hw_A)),
-     CONS (SYM (blit_sw),      BOOLEAN (info->blit_sw)),
-     CONS (SYM (blit_sw_CC),   BOOLEAN (info->blit_sw_CC)),
-     CONS (SYM (blit_sw_A),    BOOLEAN (info->blit_sw_A)),
-     CONS (SYM (blit_fill),    BOOLEAN (info->blit_fill)),
-     CONS (SYM (video_mem),    NUM_ULONG (info->video_mem)),
-     CONS (SYM (vfmt),         format));
+#define PUSHPAIR(a,b)  rv = CONS (CONS (SYM (a), b), rv)
+
+  PUSHPAIR (vfmt,         format);
+  PUSHPAIR (video_mem,    NUM_ULONG (info->video_mem));
+  PUSHPAIR (blit_fill,    BOOLEAN (info->blit_fill));
+  PUSHPAIR (blit_sw_A,    BOOLEAN (info->blit_sw_A));
+  PUSHPAIR (blit_sw_CC,   BOOLEAN (info->blit_sw_CC));
+  PUSHPAIR (blit_sw,      BOOLEAN (info->blit_sw));
+  PUSHPAIR (blit_hw_A,    BOOLEAN (info->blit_hw_A));
+  PUSHPAIR (blit_hw_CC,   BOOLEAN (info->blit_hw_CC));
+  PUSHPAIR (blit_hw,      BOOLEAN (info->blit_hw));
+  PUSHPAIR (wm_available, BOOLEAN (info->wm_available));
+  PUSHPAIR (hw_available, BOOLEAN (info->hw_available));
+
+#undef PUSHPAIR
+
+  return rv;
 #undef FUNC_NAME
 }
 
@@ -337,9 +348,9 @@ PRIMPROC
  (SCM format, SCM flags),
  doc: /***********
 Return a list of available screen dimensions for pixel
-@var{format} and @var{flags}.  Format defaults to that for
-the current screen.  Flags default to none
-(see @code{flagstash:video}).
+@var{format} and @var{flags} (@pxref{video flags}).
+Format defaults to that for
+the current screen.  Flags default to none.
 Return @code{#f} if no modes are available, @code{#t} if all are available.  */)
 {
 #define FUNC_NAME s_list_modes
@@ -358,18 +369,20 @@ Return @code{#f} if no modes are available, @code{#t} if all are available.  */)
   UNBOUND_MEANS_FALSE (flags);
   if (NOT_FALSEP (flags))
     {
+      DECLINIT_SYM2NUM_CC (2, btw->video_flags);
+
       ASSERT_INTEGER (flags, 2);
-      cflags = GSDL_FLAGS2ULONG (flags, btw->video_flags, 2);
+      cflags = FLAGS2ULONG (2, flags);
     }
 
   modes = SDL_ListModes (cformat, cflags);
 
   if (modes == (SDL_Rect**)0)
     /* Return ‘#f’ to signify no resolutions are available.  */
-    SET_FALSE (result);
+    result = BOOL_FALSE;
   else if (modes == (SDL_Rect**)-1)
     /* Return ‘#t’ to signify all resolutions are available.  */
-    SET_TRUE (result);
+    result = BOOL_TRUE;
   else
     {
       int i;
@@ -395,12 +408,13 @@ PRIMPROC
  doc: /***********
 Check to see if a particular video mode is supported.
 Args are @var{width}, @var{height}, @var{bpp} (numbers),
-and @var{flags} (see @code{flagstash:video}).
+and @var{flags} (@pxref{video flags}).
 Return @code{#f} if the mode is not supported, or a number
 indicating the bits-per-pixel of the closest available
 mode supporting @var{width} and @var{height}.  */)
 {
 #define FUNC_NAME s_video_mode_ok
+  DECLINIT_SYM2NUM_CC (4, btw->video_flags);
   Uint32 cflags = 0;
   int result;
 
@@ -409,7 +423,7 @@ mode supporting @var{width} and @var{height}.  */)
   ASSERT_INTEGER (bpp,    3);
 
   if (BOUNDP (flags))
-    cflags = GSDL_FLAGS2ULONG (flags, btw->video_flags, 4);
+    cflags = FLAGS2ULONG (4, flags);
 
   result = SDL_VideoModeOK (C_LONG (width), C_LONG (height),
                             C_LONG (bpp), cflags);
@@ -424,10 +438,11 @@ PRIMPROC
  doc: /***********
 Set the SDL video mode with @var{width},
 @var{height} and bits-per-pixel @var{bpp}.  Optional arg
-@var{flags} (see @code{flagstash:video}) is supported.
+@var{flags} (@pxref{video flags}) is supported.
 Return a new surface.  */)
 {
 #define FUNC_NAME s_set_video_mode
+  DECLINIT_SYM2NUM_CC (4, btw->video_flags);
   Uint32 cflags = 0;
 
   ASSERT_INTEGER (width,  1);
@@ -435,7 +450,7 @@ Return a new surface.  */)
   ASSERT_INTEGER (bpp,    3);
 
   if (BOUNDP (flags))
-    cflags = GSDL_FLAGS2ULONG (flags, btw->video_flags, 4);
+    cflags = FLAGS2ULONG (4, flags);
 
   RETURN_INT_SURFACE
     (SDL_SetVideoMode (C_LONG (width), C_LONG (height),
@@ -540,7 +555,7 @@ assert_colormap_copy (const char *FUNC_NAME, SCM v, int pos,
                       int *len, SDL_Color *cv)
 {
   if (! VECTORP (v)
-      || 256 < (*len = VECLENGTH (v)))
+      || 256 < (*len = VECTOR_LEN (v)))
     *len = -1;
   else
     {
@@ -548,7 +563,7 @@ assert_colormap_copy (const char *FUNC_NAME, SCM v, int pos,
 
       for (i = 0; i < *len; i++)
         {
-          SCM elem = VECREF (v, i);
+          SCM elem = VECTOR_REF (v, i);
 
           if (! COLOR_P (elem))
             {
@@ -567,45 +582,60 @@ assert_colormap_copy (const char *FUNC_NAME, SCM v, int pos,
 
 
 PRIMPROC
-(set_colors, "set-colors!", 2, 0, 0,
- (SCM surface, SCM colors),
+(set_colors, "set-colors!", 2, 1, 0,
+ (SCM surface, SCM colors, SCM start),
  doc: /***********
 Set a portion of the colormap for the 8-bit @var{surface}
-using @var{colors}, a vector of SDL-Colors.  */)
+using @var{colors}, a vector of SDL-Colors.
+Optional arg @var{start} (an integer in the range [0,255])
+specifies the portion to be modified.  It defaults to 0.  */)
 {
 #define FUNC_NAME s_set_colors
   SDL_Color ccolors[256];
-  int length;
+  int length, cstart = 0;
 
   ASSERT_SURFACE (surface, 1);
   ASSERT_COLORMAP_COPY (colors, 2, length);
+  if (BOUNDP (start))
+    {
+      ASSERT_ULONG_COPY (start, 3);
+      SCM_ASSERT_RANGE (4, start, cstart < 256);
+    }
 
   RETURN_BOOL
     (SDL_SetColors (UNPACK_SURFACE (surface),
-                    ccolors, 0, length));
+                    ccolors, cstart, length));
 #undef FUNC_NAME
 }
 
 
 PRIMPROC
-(set_palette, "set-palette", 3, 0, 0,
- (SCM surface, SCM flags, SCM colors),
+(set_palette, "set-palette", 3, 1, 0,
+ (SCM surface, SCM flags, SCM colors, SCM start),
  doc: /***********
 Set the palette of an 8-bit @var{surface}
-using @var{flags} (see @code{flagstash:palette}) and
-@var{colors}, a vector of SDL-Colors.  */)
+using @var{flags} (@pxref{palette flags}) and
+@var{colors}, a vector of SDL-Colors.
+Optional arg @var{start} (an integer in the range [0,255])
+specifies the portion to be modified.  It defaults to 0.  */)
 {
 #define FUNC_NAME s_set_palette
+  DECLINIT_SYM2NUM_CC (2, palette_flags);
   SDL_Color ccolors[256];
-  int cflags, length;
+  int cflags, length, cstart = 0;
 
   ASSERT_SURFACE (surface, 1);
   ASSERT_COLORMAP_COPY (colors, 3, length);
+  if (BOUNDP (start))
+    {
+      ASSERT_ULONG_COPY (start, 4);
+      SCM_ASSERT_RANGE (4, start, cstart < 256);
+    }
 
-  cflags   = GSDL_FLAGS2ULONG (flags, palette_flags, 2);
+  cflags   = FLAGS2ULONG (2, flags);
   RETURN_BOOL
     (SDL_SetPalette (UNPACK_SURFACE (surface),
-                     cflags, ccolors, 0, length));
+                     cflags, ccolors, cstart, length));
 #undef FUNC_NAME
 }
 
@@ -641,6 +671,8 @@ Each table is a u16 uniform vector of length 256.
 Return @code{#f} if unsuccessful.  */)
 {
 #define FUNC_NAME s_get_gamma_ramp
+  DECLARE_UV_STRUCT (/* not const */, u16, Uint16);
+
   U16_STUFF (r); SCM r;
   U16_STUFF (g); SCM g;
   U16_STUFF (b); SCM b;
@@ -649,8 +681,8 @@ Return @code{#f} if unsuccessful.  */)
 
 #define GREET(v)                                        \
   v = scm_make_u16vector (table_size, SCM_UNDEFINED);   \
-  GET_U16_PARTICULARS (v)
-#define WRITABLE_VBITS(v)  (Uint16 *) ST (v, elt)
+  GET_WRITABLE_PARTICULARS (u16, v)
+#define WRITABLE_VBITS(v)  ST (v, elt)
 
   GREET (r);
   GREET (g);
@@ -662,9 +694,9 @@ Return @code{#f} if unsuccessful.  */)
   LATER (g);
   LATER (b);
   if (0 > rv)
-    RETURN_FALSE;
+    return BOOL_FALSE;
   else
-    RETURN_LIST3 (r, g, b);
+    return LIST3 (r, g, b);
 
 #undef WRITABLE_VBITS
 #undef GREET
@@ -788,6 +820,31 @@ Return the mapped components as an unsigned integer.  */)
 }
 
 
+PRIMPROC
+(pixel_rgb, "pixel-rgb", 2, 0, 0,
+ (SCM pixel,
+  SCM format),
+ doc: /***********
+Return RGB info from @var{pixel} in the specified pixel @var{format}
+as three values: @code{r}, @code{g} and @code{b} (all integers).  */)
+{
+#define FUNC_NAME s_pixel_rgb
+  Uint8 r, g, b;
+
+  ASSERT_INTEGER (pixel, 1);
+  ASSERT_PIXEL_FORMAT (format, 2);
+
+  SDL_GetRGB (C_ULONG (pixel), UNPACK_PIXEL_FORMAT (format),
+              &r, &g, &b);
+
+  RETURN_VALUES3
+    (NUM_ULONG (r),
+     NUM_ULONG (g),
+     NUM_ULONG (b));
+#undef FUNC_NAME
+}
+
+
 DECLARE_SIMPLE_SYM (r);
 DECLARE_SIMPLE_SYM (g);
 DECLARE_SIMPLE_SYM (b);
@@ -798,6 +855,9 @@ PRIMPROC
  (SCM pixel,
   SCM format),
  doc: /***********
+NB: This procedure is obsoleted by @code{pixel-rgb}
+and @strong{will be removed} after 2013-12-31.
+
 Get RGB values from @var{pixel} in the specified pixel
 @var{format}.  Return an alist with keys @code{r}, @code{g}
 and @code{b}, with red, green and blue values (numbers),
@@ -812,17 +872,45 @@ respectively.  */)
   SDL_GetRGB (C_ULONG (pixel), UNPACK_PIXEL_FORMAT (format),
               &r, &g, &b);
 
-  RETURN_LIST3 (CONS (SYM (r), NUM_ULONG (r)),
+  return LIST3 (CONS (SYM (r), NUM_ULONG (r)),
                 CONS (SYM (g), NUM_ULONG (g)),
                 CONS (SYM (b), NUM_ULONG (b)));
 #undef FUNC_NAME
 }
 
 
+PRIMPROC
+(pixel_rgba, "pixel-rgba", 2, 0, 0,
+ (SCM pixel, SCM format),
+ doc: /***********
+Return RGBA info from @var{pixel} in the specified pixel @var{format} as
+four values: @code{r}, @code{g}, @code{b} and @code{a} (all integers).  */)
+{
+#define FUNC_NAME s_pixel_rgba
+  Uint8 r, g, b, a;
+
+  ASSERT_INTEGER (pixel, 1);
+  ASSERT_PIXEL_FORMAT (format, 2);
+
+  SDL_GetRGBA (C_ULONG (pixel), UNPACK_PIXEL_FORMAT (format),
+               &r, &g, &b, &a);
+
+  RETURN_VALUES4
+    (NUM_ULONG (r),
+     NUM_ULONG (g),
+     NUM_ULONG (b),
+     NUM_ULONG (a));
+#undef FUNC_NAME
+}
+
+
 PRIMPROC
 (get_rgba, "get-rgba", 2, 0, 0,
  (SCM pixel, SCM format),
  doc: /***********
+NB: This procedure is obsoleted by @code{pixel-rgba}
+and @strong{will be removed} after 2013-12-31.
+
 Get RGBA values from @var{pixel} in the specified pixel
 @var{format}.  Return an alist with keys @code{r}, @code{g},
 @code{b} and @code{a}, with red, green, blue and alpha values
@@ -837,7 +925,7 @@ Get RGBA values from @var{pixel} in the specified pixel
   SDL_GetRGBA (C_ULONG (pixel), UNPACK_PIXEL_FORMAT (format),
                &r, &g, &b, &a);
 
-  RETURN_LIST4 (CONS (SYM (r), NUM_ULONG (r)),
+  return LIST4 (CONS (SYM (r), NUM_ULONG (r)),
                 CONS (SYM (g), NUM_ULONG (g)),
                 CONS (SYM (b), NUM_ULONG (b)),
                 CONS (SYM (a), NUM_ULONG (a)));
@@ -885,7 +973,7 @@ to the display format.  Return @code{#f} if not successful.  */)
   csurface = SDL_DisplayFormat (UNPACK_SURFACE (surface));
 
   if (! csurface)
-    RETURN_FALSE;
+    return BOOL_FALSE;
 
   RETURN_NEW_SURFACE (csurface);
 #undef FUNC_NAME
@@ -908,7 +996,7 @@ if not successful.  */)
   csurface = SDL_DisplayFormatAlpha (UNPACK_SURFACE (surface));
 
   if (! csurface)
-    RETURN_FALSE;
+    return BOOL_FALSE;
 
   RETURN_NEW_SURFACE (csurface);
 #undef FUNC_NAME
@@ -1100,6 +1188,24 @@ If @var{icon} is not specified, use @var{title} by default.  */)
 }
 
 
+PRIMPROC
+(caption_ti, "caption-ti", 0, 0, 0,
+ (void),
+ doc: /***********
+Return display-window caption as two values: @code{title}
+and @code{icon} (both strings, or @code{#f} if not set).  */)
+{
+#define FUNC_NAME s_caption_ti
+  char *title, *icon;
+
+  SDL_WM_GetCaption (&title, &icon);
+  RETURN_VALUES2
+    (title ? STRING (title) : BOOL_FALSE,
+     icon  ? STRING (icon)  : BOOL_FALSE);
+#undef FUNC_NAME
+}
+
+
 DECLARE_SIMPLE_SYM (title);
 DECLARE_SIMPLE_SYM (icon);
 
@@ -1107,6 +1213,9 @@ PRIMPROC
 (wm_get_caption, "get-caption", 0, 0, 0,
  (void),
  doc: /***********
+NB: This procedure is obsoleted by @code{caption-ti}
+and @strong{will be removed} after 2013-12-31.
+
 Return an alist with keys @code{title} and @code{icon}
 and values the title-bar and icon name (or @code{#f}) of the display
 window, respectively.  */)
@@ -1115,7 +1224,7 @@ window, respectively.  */)
   char *title, *icon;
 
   SDL_WM_GetCaption (&title, &icon);
-  RETURN_LIST2 (CONS (SYM (title), title ? STRING (title) : BOOL_FALSE),
+  return LIST2 (CONS (SYM (title), title ? STRING (title) : BOOL_FALSE),
                 CONS (SYM (icon),  icon  ? STRING (icon)  : BOOL_FALSE));
 #undef FUNC_NAME
 }
@@ -1177,10 +1286,6 @@ Return @code{#t} if successful.  */)
 }
 
 
-DECLARE_SIMPLE_SYM (query);
-DECLARE_SIMPLE_SYM (off);
-DECLARE_SIMPLE_SYM (on);
-
 PRIMPROC
 (wm_grab_input, "grab-input", 0, 1, 0,
  (SCM mode),
@@ -1191,30 +1296,21 @@ of grab, one of @code{query} (the default),
 @code{off} or @code{on}.  */)
 {
 #define FUNC_NAME s_wm_grab_input
-  if (UNBOUNDP (mode))
-    mode = SYM (query);
-  else
-    ASSERT_SYMBOL (mode, 1);
-  if (! (EQ (mode, SYM (query)) ||
-         EQ (mode, SYM (off)) ||
-         EQ (mode, SYM (on))))
-    SCM_MISC_ERROR ("bad mode: ~S", CONS (mode, SCM_EOL));
+  DECLINIT_SYM2NUM_CC (1, grab_modes);
+  int cmode = BOUNDP (mode)
+    ? ENUM2LONG (1, mode)
+    : SDL_GRAB_QUERY;
 
-  return (SDL_GRAB_ON == SDL_WM_GrabInput (EQ (mode, SYM (query))
-                                           ? SDL_GRAB_QUERY
-                                           : (EQ (mode, SYM (on))
-                                              ? SDL_GRAB_ON
-                                              : SDL_GRAB_OFF))
-          ? SYM (on)
-          : SYM (off));
+  return btw->long2enum (SDL_WM_GrabInput (cmode), grab_modes);
 #undef FUNC_NAME
 }
 
 
 
-#include "video.c"
-#include "palette.c"
-#include "overlay.c"
+#include "k/video.c"
+#include "k/palette.c"
+#include "k/overlay.c"
+#include "k/grabmode.c"
 
 void
 gsdl_init_video (void)
@@ -1234,17 +1330,23 @@ gsdl_init_video (void)
            free_yuv_overlay,
            NULL);
 
-  /* video flags */
-  btw->video_flags = btw->make_flagstash (&vid_flagstash);
+  {
+    kf_init_t allf[] = {
+      { &btw->video_flags, &vid_flagstash },
+      { &palette_flags, &pal_flagstash },
+      { &gsdl_overlay_formats, &ov_flagstash }
+    };
 
-  /* palette flags */
-  palette_flags = btw->make_flagstash (&pal_flagstash);
+    REGISTER_KF_V (allf);
+  }
 
-  /* yuv overlay formats */
-  gsdl_overlay_formats = btw->make_flagstash (&ov_flagstash);
+  {
+    kp_init_t allp[] = {
+      { &grab_modes, &grabmode_kp }
+    };
 
-  /* GL constants */
-  gl_enums = DEFINE_ENUM ("gl-enums", gl_eback);
+    REGISTER_KP_V (allp);
+  }
 
 #include "sdlvideo.x"
 }
